' RFID_API.bas
' Written for the ZX24a MicroControler in Zbasic version 2.3.0
' October 2007, Lewis Balentine, Jupiter, Florida
' ------------------------------------------------------------
Option Explicit
' This module is intended to read the BIT stream from
' a HID 125 KHz RFID reader. The reader has five wires 
' that are connected as follows:
' 	+ 5 Volts, Red
' 	ground, Black
' 	Signal Line zero, White (connect to pin 6)
' 	Signal Line one, Green  (connect to pin 11)
' 	N/C = LED, Orange
' No additional hardware is required.
'
' The reader holds each line at 5 volts. 
' When a card is read:
'   it lowers the Signal line zero for a zero.
'   it lowers the Signal line one for a one.
'   the timing and number of pulses varies by manufacturer and model.
'   the generic parameters are:
'   	each pulse is between 20-100 uS long 
'		pulse Interval between 200uS and 20mS).
'   	less than 60 pulses.
'   	Common cards formats are 26 and 34 pulses.
'
' Sample Data
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' 26 Bit card with 9 bit facility code and 15 bit ID code and start/end bits
' 1 234567890 123456789012345 6  (bit numbers)
' 0 001001000 101100011110001 1
'             101100011110001 = 22769 (ID)
'   001001000 = 72 (Facility code)
'
' 26 Bit card with 16 bit facility code and 16 bit ID code and start/end bits
' 1 2345678901234567 8901234567890123 4  (bit numbers)
' 0 0000001000010000 1000000101001101 0
'                    1000000101001101 = 33101  (ID)
'   0000001000010000 = 528 (Facility code)
'
' 26 Bit card with 16 bit facility code and 16 bit ID code and start/end bits
' 1 2345678901234567 8901234567890123 4  (bit numbers)
' 0 0000001000010000 1000000011101001 0
'                    1000000011101001 = 33001  (ID)
'   0000001000010000 = 528 (Facility code)
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'   The ZX24a has pin change interupts on all IO pins. It also
'   has falling edge and rising edge interupt for the port 
'   Interupt pins:
'      Interupt 0 on Port C (bit 6) is Pin 6
'      Interupt 1 on Port C (bit 1) is Pin 11
'      Interupt 2 on Port A (bit 2) is Pin 18
'	We are going to use the falling edge on pins 6 and 11.
'	(or the prallax pro dev board that is: X16.1 and X16.6)
'
'   By Imperical testing we get one pulse per RTC tick.
'	So we will assume that after one quarter second we have the 
'   complete stream.
Private Const RfidDelay as Single = 0.25
Private Const RfidMaxBits as Byte = 60
'
' Define Que for pulses
' Stuffing bits would require additional code in the interupt routines.
' We are trying to keep them as small and fast as possible.
' Thus we will store the data as bytes and convert it after we read the stream.
Private RFIDQue  (1 To RfidMaxBits) As Byte 	' Holds zero or one for each pulse
Private RfidIdx as Byte 	   					' Holds pointer to current pulse
'
'
' Stack           Min.   Act.
'   Size  Margin   Size   Size  Module.TaskName            
'  -----  ------  -----  -----  -------------------------- 
'     23      10     33     35  RFID_API.RFIDZeroTask      
'     23      10     33     35  RFID_API.RFIDoneTask       
' Stack sizes adjusted after compile ...
Private RFIDZeroTaskStack (1 to 35) as Byte 
Private RFIDOneTaskStack (1 to 35) as Byte	 

'----------------------------------------------
Public Sub RFIDRead ()
	Call InitRFIDQue
	CallTask RFIDZeroTask, RFIDZeroTaskStack
	CallTask RFIDOneTask, RFIDOneTaskStack
	Call PutPin (25, zxOutputHigh)					' Turn Off OnBoard LED 
	Do
		' Using the delay here allows other tasks to run
		Call Delay (RfidDelay)
		If RfidIdx > 0 then
			Call Delay (RfidDelay)					' Wait to get the entire stream
			Exit Do
		Else
			Call PutPin (25, zxOutputToggle)		' Toggle Onboard LED
		End If
	Loop
	Call PutPin (25, zxOutputHigh)					' Turn Off OnBoard LED 
	Call ExitTask (RFIDZeroTaskStack)
	Call ExitTask (RFIDOneTaskStack)
	' Debug.Print CStr(RfidIdx);", ";RFIDStr			' For Debugging only
	
End Sub
'----------------------------------------------
Private Sub RFIDZeroTask ()
	Call PutPin ( 6, zxInputPullUp)					' Configure for Input pull-up
	Do
		' Wait for the pin to go Low
		Call WaitForInterrupt (zxPinFallingEdge, WaitInt0)
		RfidIdx = RfidIdx+1							' Incremet the Index
		RFIDQue(RfidIdx)=0							' put zero in the Que
	Loop
End Sub

'------------------------------------------------
Private Sub RFIDoneTask ()
	Call PutPin (11, zxInputPullUp)					' Configure for Input pull-up
	Do
		' Wait for the pin to go Low
		Call WaitForInterrupt (zxPinFallingEdge, WaitInt1)
		RfidIdx = RfidIdx+1							' Incremet the Index
		RFIDQue(RfidIdx)=1							' put one in the Que
	Loop
End Sub

'----------------------------------------------
Private Sub InitRFIDQue ()
	Dim Idx as Byte
	' We fill all bits just to make sure we do not miss a pulse
	For Idx = 1 to RfidMaxBits
		RFIDQue (Idx)=255
	Next Idx
	RfidIdx=0
End Sub

'----------------------------------------------
Public Function RFIDStr () as String
	' Returns a String comprised of ones and zeros
	' that represents the content of the RFID que
   	Dim I as Byte
	Dim S as String
	S=""
	For I= 1 to RfidIdx						' comment out of production version
		S=S & CStr(RFIDQue(I))
	Next I
	RFIDStr=S
End Function

'----------------------------------------------
Public Function RfIDcode () as String
	' This is based on the cards that I am working with
	' Others should revise this routine to fit their own applications
	Select Case RfidIdx
		Case 26
			RfIDcode=Cstr(RfidDec(11,25))
		Case 34
			RfIDcode=Cstr(RfidDec(18,33))
		Case Else
			RfIDcode=""
	End Select
End Function

'----------------------------------------------
Public Function RfIDFacility () as String
	' This is based on the cards that I am working with
	' Others should revise this routine to fit their own applications
	Select Case RfidIdx
		Case 26
			RfIDFacility=Cstr(RfidDec(2,10))
		Case 34
			RfIDFacility=Cstr(RfidDec(1,17))
		Case Else
			RfIDFacility=""
	End Select
End Function
		
'----------------------------------------------
Public Function RfidDec (byval StartBit as Byte, _
	byval StopBit as Byte ) as UnsignedLong
	' Returns a Decimal 32 Bit Long value for the 
	' specified bits from the RFID que
	'
	' Online Binary/decimal converter used to verify conversions
	' http://acc6.its.brooklyn.cuny.edu/~gurwitz/core5/nav2tool.html
	'
	' There is not given standard for the facility code vs ID code.
	' Therefore the calling application must communicate the bits of interest.
	' Start from the least significant and move to the highest to convert to a decimal.
	Dim I as Byte
	Dim J as UnsignedLong
	Dim W as UnsignedLong
	I=0
	J=1
	W=0
	RfidDec=0
	if StopBit > RfidIdx then 
		Exit Function
	End If
	if StartBit < 1 then 
		Exit Function
	End If
	For I= StopBit to StartBit Step -1
		IF RFIDQue(I)<>0 then
			W=W or J
		End If
		J=Shl(J,1)							' same as multiplying by 2
	Next I
	RfidDec=W
End Function




